#version 150
precision mediump float;

//#define HEIGHT 768
#define PI 3.141692

uniform sampler2DRect tex0;
out vec4 outputColor;
in vec2 texCoordVarying;
uniform float threshold = 0.5;
uniform float offset = 0;
uniform float steps = 1;
uniform float _mix = 1;
uniform float forceSaturation = 0;
uniform float bwSmoothing = 0;//0.1; // 0 <= smoothing <= 0.5

vec3 rgb2hsv(vec4 c){
    vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
    vec4 p = mix(vec4(c.bg, K.wz), vec4(c.gb, K.xy), step(c.b, c.g));
    vec4 q = mix(vec4(p.xyw, c.r), vec4(c.r, p.yzx), step(p.x, c.r));
    
    float d = q.x - min(q.w, q.y);
    float e = 1.0e-10;
    return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
}

vec4 hsv2rgb(vec3 c, float a){
    vec4 K = vec4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
    vec3 p = abs(fract(c.xxx + K.xyz) * 6.0 - K.www);
    return vec4(c.z * mix(K.xxx, clamp(p - K.xxx, 0.0, 1.0), c.y), a);
}

void main(){
    vec4 c = texture(tex0, texCoordVarying);
    vec4 cbw = c;
    if(steps > 0){
        float v;
        float pos = mod(offset + steps * ((c.r + c.g + c.b) / 3.0 ), 1);

//        if(abs(pos) < bwSmoothing)
//        {
//            v = sin((pos) / bwSmoothing * 3.14);
////            v =  (pos) / bwSmoothing;
//        }
//        else
//        {
            v = 1 - step(pos , threshold);
//        }
        cbw = vec4(v,v,v,c.a);
    }else{
        vec3 hsv = rgb2hsv(c);
        float value = hsv.z;
        float i = (c.r + c.g + c.b) / 3.0;
        float v = mod(offset - steps * i, 1) / 1.0;
        cbw = hsv2rgb(vec3(v, forceSaturation==1 ? i : hsv.y, value), c.a); //it's so fun to tweak : like assigning the hue to the value !
    }
    
    outputColor = mix(c, cbw, _mix*_mix);
    
}
