#version 150
precision mediump float;

#define PI 3.141692

uniform sampler2DRect tex0;
out vec4 outputColor;
in vec2 texCoordVarying;
uniform float reTint = 0;
uniform float reSaturate = 0;

uniform float WIDTH = 1;
uniform float HEIGHT = 1;

uniform float saturation = 0.8;
uniform float hue=0;
uniform float gradient=0.17;
uniform float forceBrightness = -1;

uniform float useTexture = 0;
uniform float texOffX = 0;
uniform float texOffY = 0;
uniform sampler2DRect tex1;

vec3 rgb2hsv(vec4 c){
    vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
    vec4 p = mix(vec4(c.bg, K.wz), vec4(c.gb, K.xy), step(c.b, c.g));
    vec4 q = mix(vec4(p.xyw, c.r), vec4(c.r, p.yzx), step(p.x, c.r));
    
    float d = q.x - min(q.w, q.y);
    float e = 1.0e-10;
    return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
}

vec4 hsv2rgb(vec3 c){
    vec4 K = vec4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
    vec3 p = abs(fract(c.xxx + K.xyz) * 6.0 - K.www);
    return vec4(c.z * mix(K.xxx, clamp(p - K.xxx, 0.0, 1.0), c.y), 1);
}

float getAdjustedBrightness(vec4 c){
    float perceivedLightness = (0.2126*c.r + 0.7152*c.g + 0.0722*c.b);
    float br = 1.07 - perceivedLightness/3.5;
    return br;
}

void main(){
    vec2 p = texCoordVarying;
    
    vec4 brushColor = texture(tex0, p); //brushColor
    float a = brushColor.a;

    vec4 texColor = vec4(1,1,1,1);
    vec4 chromaColor = vec4(1,1,1,1);
    
    float brushIntensity = 1;

    if(useTexture>0){
        texColor = texture(tex1, texCoordVarying + vec2(texOffX, texOffY));
        brushIntensity = (brushColor.r + brushColor.g + brushColor.b) / 1.5;
          //brushColor.xyz = texColor.xyz * 1;
          brushColor.xyz = texColor.xyz * brushIntensity;
    }

    
    vec3 reTintedColor_hsv = rgb2hsv(brushColor);
    
    //retint : 0 : classic texture hue
    //retint :: 1 : brush hue
    
    reTintedColor_hsv.y = saturation/255.0;
    
    if(reTint!=0){
        if(gradient<1){
            reTintedColor_hsv.x = mix(hue, hue + gradient, p.x / WIDTH);
        }else{
            if(gradient < 2){
                reTintedColor_hsv.x = mix(hue, hue + gradient-1, p.y / HEIGHT);
            }else{
                reTintedColor_hsv.x = mix(hue, hue + gradient-2, (p.x / WIDTH + p.y / HEIGHT) / 2.0);
            }
        }
    }
    
   // if(forceBrightness>=0){
   //     reTintedColor_hsv.z = forceBrightness;
   // }
    
    reTintedColor_hsv.y = saturation;
    if(reSaturate!=0)
        reTintedColor_hsv.y += reSaturate;


    
    if(reTint>0){
        chromaColor = hsv2rgb(reTintedColor_hsv);
        outputColor = mix(brushColor, chromaColor, reTint);
    }
    else
    {
        outputColor = brushColor;
    }
    


    if(saturation>0 && reTint!=0){
     //   float correction = getAdjustedBrightness(outputColor);
     //   outputColor *= vec4(correction,correction,correction,1);
    }

   if(forceBrightness>=0){
        //outputColor.xyz = mix(outputColor.xyz*forceBrightness, outputColor.xyz, 0.5);
       //outputColor.xyz = outputColor.xyz*forceBrightness;
        a *= forceBrightness;
    }
    
    
    outputColor.a = a;
}
