#version 150
precision mediump float;

uniform sampler2DRect tex0;
uniform sampler2DRect depthMap;
out vec4 outputColor;
in vec2 texCoordVarying;
uniform int WIDTH = 1280;
uniform int HEIGHT = 720;
uniform float intensity = 0;
uniform int mode = 2;

vec3 rgb2hsv(vec4 c){
    vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
    vec4 p = mix(vec4(c.bg, K.wz), vec4(c.gb, K.xy), step(c.b, c.g));
    vec4 q = mix(vec4(p.xyw, c.r), vec4(c.r, p.yzx), step(p.x, c.r));
    
    float d = q.x - min(q.w, q.y);
    float e = 1.0e-10;
    return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
}

void main(){
    vec2 p = texCoordVarying - vec2(WIDTH/2,HEIGHT/2);
    
    vec4 d = texture(depthMap, vec2(p.x, HEIGHT-p.y) + vec2(WIDTH/2,-HEIGHT/2));
    vec3 hsv = rgb2hsv(d);
    
    if(mode==0){
        p.y -= (hsv.z-0.5)*intensity*HEIGHT;
    }else if(mode==1){
        p.x -= (hsv.z-0.5)*intensity*HEIGHT;
    }else if(mode==2){
        float d = sqrt(p.x*p.x + p.y*p.y);
        float a = atan(p.y,p.x);
        d *=  1 + (hsv.z-0.5)*intensity;
        p.x = d*cos(a);
        p.y = d*sin(a);
    }
    
    vec4 t = texture(depthMap, vec2(p.x, HEIGHT - (p.y)) + vec2(WIDTH/2,-HEIGHT/2));

    if((hsv.z-0.5) > 0 && (t.z-0.5 < 0))
        {
            outputColor = vec4(0,0,0,1);
            return;
        }
    
    if((hsv.z-0.5) < 0 && (t.z-0.5 > 0))
        {
            outputColor = vec4(0,0,0,1);
            return;
        }
    
    
    outputColor = texture(tex0, p + vec2(WIDTH/2,HEIGHT/2));
    
}

