#version 150

uniform sampler2DRect tex0;
in vec2 texCoordVarying;
out vec4 outputColor;
uniform float circle = 0;
uniform float WIDTH = 1280;
uniform float HEIGHT = 720;
uniform int mode = 1;

vec3 rgb2hsv(vec4 c){
    vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
    vec4 p = mix(vec4(c.bg, K.wz), vec4(c.gb, K.xy), step(c.b, c.g));
    vec4 q = mix(vec4(p.xyw, c.r), vec4(c.r, p.yzx), step(p.x, c.r));
    
    float d = q.x - min(q.w, q.y);
    float e = 1.0e-10;
    return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
}

vec4 hsv2rgb(vec3 c){
    vec4 K = vec4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
    vec3 p = abs(fract(c.xxx + K.xyz) * 6.0 - K.www);
    return vec4(c.z * mix(K.xxx, clamp(p - K.xxx, 0.0, 1.0), c.y), 1);
}

void main()
{
    vec4 c = texture(tex0, texCoordVarying);
    
    if(circle!=0){
        float x = texCoordVarying.x-WIDTH/2;
        float y = texCoordVarying.y-HEIGHT/2;
        float d = (x*x + y*y)/(HEIGHT/2*HEIGHT/2);
        if((circle>0 && d > circle) || (circle<0 && d < -circle)){
            c.r = 1 -  c.r;
            c.b = 1 -  c.b;
            c.g = 1 -  c.g;
        }
    }else{
        if(mode==1){ // keep the hue
            vec3 h = rgb2hsv(c);
            h.x = h.x + 0.5;
            c = hsv2rgb(h);
            
            c.r = 1 -  c.r;
            c.b = 1 -  c.b;
            c.g = 1 -  c.g;
        }else{
            if(mode==2){
                c.r = 1 -  c.r;
                c.b = 1 -  c.b;
                c.g = 1 -  c.g;
            }
            else{
                if(mode==3){
                    c.r = 0;
                    c.b = 0;
                    c.g = 0;
                }
            }
        }
    }
    outputColor = c;
    }