#version 150
precision mediump float;

#define M_PI 3.1415

uniform sampler2DRect tex0;
out vec4 outputColor;
in vec2 texCoordVarying;
uniform float WIDTH=1000;
uniform float HEIGHT=1000;
uniform float intensity=0;
uniform float evolution=0;
uniform float scale=0;
uniform float freq=7;
uniform float persistance = .5;
uniform int mode = 0;
uniform int hue = 0;
uniform float hueZ = 1; //good value : 5
uniform int octaves = 3;

vec3 rgb2hsv(vec4 c){
    vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
    vec4 p = mix(vec4(c.bg, K.wz), vec4(c.gb, K.xy), step(c.b, c.g));
    vec4 q = mix(vec4(p.xyw, c.r), vec4(c.r, p.yzx), step(p.x, c.r));
    
    float d = q.x - min(q.w, q.y);
    float e = 1.0e-10;
    return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
}

vec4 hsv2rgb(vec3 c){
    vec4 K = vec4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
    vec3 p = abs(fract(c.xxx + K.xyz) * 6.0 - K.www);
    return vec4(c.z * mix(K.xxx, clamp(p - K.xxx, 0.0, 1.0), c.y), 1);
}

float rand(vec2 c){
    return fract(sin(dot(c ,vec2(12.9898,78.233))) * 43758.5453);
}

float noise(vec2 p, float freq ){
    float unit = WIDTH/freq;
    vec2 ij = floor(p/unit);
    vec2 xy = mod(p,unit)/unit;
    xy = .5*(1.-cos(M_PI*xy));
    float a = rand((ij+vec2(0.,0.)));
    float b = rand((ij+vec2(1.,0.)));
    float c = rand((ij+vec2(0.,1.)));
    float d = rand((ij+vec2(1.,1.)));
    float x1 = mix(a, b, xy.x);
    float x2 = mix(c, d, xy.x);
    return mix(x1, x2, xy.y);
}

float pNoise(vec2 p, float res){
    float n = 0.;
    float normK = 0.;
    float f = 4.;
    float amp = 1.;
    int iCount = 0;
    for (int i = 0; i<50; i++){
        n+=amp*noise(p, f);
        f*=2.;
        normK+=amp;
        amp*=persistance;
        if (iCount == int(res)) break;
        iCount++;
    }
    float nf = n/normK; //offset hack for better centering
    return (nf*nf*nf*nf) ;
}


void main(){
   vec2 p = texCoordVarying - vec2(WIDTH/2, HEIGHT/2);
//    vec2 p = texCoordVarying;
    float f = abs(freq);
    
    float ox = 0, oy = 0;
    
    if(mode<=2){ //cartesian
        if(mode==0 || mode==1)
            ox = (intensity * HEIGHT * pNoise(p + vec2(WIDTH*evolution,WIDTH), f));
        if(mode==0 || mode==2)
            oy =  (intensity * HEIGHT * pNoise(p + vec2(WIDTH,WIDTH*evolution), f));
        
            p.x += ox;
            p.y += oy;
            p.x -= WIDTH*step(WIDTH, p.x);
            p.y -= HEIGHT*step(HEIGHT, p.y);

        
    }else{ // polar
        float a = atan(p.y,p.x);
        float d = length(p);
        if(mode==3){
            //            d += intensity * 10 * HEIGHT * getNoise(vec2(p.x, p.y), (evolution)*vec2(WIDTH,WIDTH), f*1000);
            d += intensity * HEIGHT * pNoise(vec2(a, d) + vec2(WIDTH*evolution,WIDTH), f*1000);
        }else if(mode==4){
            //            a += intensity * 10 * 3.14 * getNoise(vec2(p.x, p.y), (evolution)*vec2(WIDTH,WIDTH), f*1000);
            a += intensity * 3.14 * pNoise(vec2(a, d) + vec2(WIDTH,WIDTH*evolution), f*1000);
        }
        p.x = d*cos(a);
        p.y = d*sin(a);
    }
    

    
    //    if(hue>0){
    //        vec4 cA = texture(tex0, texCoordVarying);
    //        vec4 cB = texture(tex0, p);
    //        vec3 hA = rgb2hsv(cA);
    //        vec3 hB = rgb2hsv(cB);
    //        hA.x = hB.x;
    //        hA.y = hB.y;
    //        outputColor = hsv2rgb(hA);
    //    }
    //    else
    if (hueZ>0)
    {
        vec4 cB = texture(tex0, p + vec2(WIDTH/2, HEIGHT/2));
        vec3 hB = rgb2hsv(cB);
        hB.x = mod(hB.x + intensity * 10 * hueZ  * 5 * (
                                                        1
                                                        +
                                                        pNoise(vec2(p.x, p.y) + (evolution)*vec2(WIDTH,WIDTH), f*1000)
                                                        )/2.0, 1);
        outputColor = hsv2rgb(hB);
    }
    else
        outputColor = texture(tex0, p + vec2(WIDTH/2, HEIGHT/2));
}

