#version 150
precision mediump float;

#define M_PI 3.1415

uniform sampler2DRect tex0;
out vec4 outputColor;
in vec2 texCoordVarying;
uniform float WIDTH=1000;
uniform float HEIGHT=1000;
uniform float intensity=0;
uniform float evolution=0;
uniform float freq=7;
uniform float persistance = .5;
uniform int mode = 0;
uniform int hue = 0;
uniform float hueZ = 1; //good value : 5
uniform int octaves = 3;

float rand(vec2 c){
    return fract(sin(dot(c ,vec2(12.9898,78.233))) * 43758.5453);
}

float noise(vec2 p, float freq ){
    float unit = WIDTH/freq;
    vec2 ij = floor(p/unit);
    vec2 xy = mod(p,unit)/unit;
    xy = .5*(1.-cos(M_PI*xy));
    float a = rand((ij+vec2(0.,0.)));
    float b = rand((ij+vec2(1.,0.)));
    float c = rand((ij+vec2(0.,1.)));
    float d = rand((ij+vec2(1.,1.)));
    float x1 = mix(a, b, xy.x);
    float x2 = mix(c, d, xy.x);
    return mix(x1, x2, xy.y);
}

float pNoise(vec2 p, float res){
    float n = 0.;
    float normK = 0.;
    float f = 4.;
    float amp = 1.;
    int iCount = 0;
    for (int i = 0; i<50; i++){
        n+=amp*noise(p, f);
        f*=2.;
        normK+=amp;
        amp*=persistance;
        if (iCount == int(res)) break;
        iCount++;
    }
    float nf = n/normK;
    return nf*nf*nf*nf;
}




vec4 permute(vec4 x){return mod(((x*34.0)+1.0)*x, 289.0);}
vec4 taylorInvSqrt(vec4 r){return 1.79284291400159 - 0.85373472095314 * r;}
vec3 fade(vec3 t) {return t*t*t*(t*(t*6.0-15.0)+10.0);}

float cnoise(vec3 P){
    vec3 Pi0 = floor(P); // Integer part for indexing
    vec3 Pi1 = Pi0 + vec3(1.0); // Integer part + 1
    Pi0 = mod(Pi0, 289.0);
    Pi1 = mod(Pi1, 289.0);
    vec3 Pf0 = fract(P); // Fractional part for interpolation
    vec3 Pf1 = Pf0 - vec3(1.0); // Fractional part - 1.0
    vec4 ix = vec4(Pi0.x, Pi1.x, Pi0.x, Pi1.x);
    vec4 iy = vec4(Pi0.yy, Pi1.yy);
    vec4 iz0 = Pi0.zzzz;
    vec4 iz1 = Pi1.zzzz;
    
    vec4 ixy = permute(permute(ix) + iy);
    vec4 ixy0 = permute(ixy + iz0);
    vec4 ixy1 = permute(ixy + iz1);
    
    vec4 gx0 = ixy0 / 7.0;
    vec4 gy0 = fract(floor(gx0) / 7.0) - 0.5;
    gx0 = fract(gx0);
    vec4 gz0 = vec4(0.5) - abs(gx0) - abs(gy0);
    vec4 sz0 = step(gz0, vec4(0.0));
    gx0 -= sz0 * (step(0.0, gx0) - 0.5);
    gy0 -= sz0 * (step(0.0, gy0) - 0.5);
    
    vec4 gx1 = ixy1 / 7.0;
    vec4 gy1 = fract(floor(gx1) / 7.0) - 0.5;
    gx1 = fract(gx1);
    vec4 gz1 = vec4(0.5) - abs(gx1) - abs(gy1);
    vec4 sz1 = step(gz1, vec4(0.0));
    gx1 -= sz1 * (step(0.0, gx1) - 0.5);
    gy1 -= sz1 * (step(0.0, gy1) - 0.5);
    
    vec3 g000 = vec3(gx0.x,gy0.x,gz0.x);
    vec3 g100 = vec3(gx0.y,gy0.y,gz0.y);
    vec3 g010 = vec3(gx0.z,gy0.z,gz0.z);
    vec3 g110 = vec3(gx0.w,gy0.w,gz0.w);
    vec3 g001 = vec3(gx1.x,gy1.x,gz1.x);
    vec3 g101 = vec3(gx1.y,gy1.y,gz1.y);
    vec3 g011 = vec3(gx1.z,gy1.z,gz1.z);
    vec3 g111 = vec3(gx1.w,gy1.w,gz1.w);
    
    vec4 norm0 = taylorInvSqrt(vec4(dot(g000, g000), dot(g010, g010), dot(g100, g100), dot(g110, g110)));
    g000 *= norm0.x;
    g010 *= norm0.y;
    g100 *= norm0.z;
    g110 *= norm0.w;
    vec4 norm1 = taylorInvSqrt(vec4(dot(g001, g001), dot(g011, g011), dot(g101, g101), dot(g111, g111)));
    g001 *= norm1.x;
    g011 *= norm1.y;
    g101 *= norm1.z;
    g111 *= norm1.w;
    
    float n000 = dot(g000, Pf0);
    float n100 = dot(g100, vec3(Pf1.x, Pf0.yz));
    float n010 = dot(g010, vec3(Pf0.x, Pf1.y, Pf0.z));
    float n110 = dot(g110, vec3(Pf1.xy, Pf0.z));
    float n001 = dot(g001, vec3(Pf0.xy, Pf1.z));
    float n101 = dot(g101, vec3(Pf1.x, Pf0.y, Pf1.z));
    float n011 = dot(g011, vec3(Pf0.x, Pf1.yz));
    float n111 = dot(g111, Pf1);
    
    vec3 fade_xyz = fade(Pf0);
    vec4 n_z = mix(vec4(n000, n100, n010, n110), vec4(n001, n101, n011, n111), fade_xyz.z);
    vec2 n_yz = mix(n_z.xy, n_z.zw, fade_xyz.y);
    float n_xyz = mix(n_yz.x, n_yz.y, fade_xyz.x);
    float t = 2.2 * n_xyz;
    return t;
}


vec3 rgb2hsv(vec4 c){
    vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
    vec4 p = mix(vec4(c.bg, K.wz), vec4(c.gb, K.xy), step(c.b, c.g));
    vec4 q = mix(vec4(p.xyw, c.r), vec4(c.r, p.yzx), step(p.x, c.r));
    
    float d = q.x - min(q.w, q.y);
    float e = 1.0e-10;
    return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
}

vec4 hsv2rgb(vec3 c){
    vec4 K = vec4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
    vec3 p = abs(fract(c.xxx + K.xyz) * 6.0 - K.www);
    return vec4(c.z * mix(K.xxx, clamp(p - K.xxx, 0.0, 1.0), c.y), 1);
}

float getNoise(vec3 p){
    return 1*cnoise(p)
    + 0.33*cnoise(p*vec3(3,3,3))
    + 0.12*cnoise(p*vec3(9,9,9));
}

void main(){
    vec2 p = texCoordVarying;
    float f = 0.001*abs(freq);

    vec3 hsv = rgb2hsv(texture(tex0, p));
    if((hsv.x < 0.2 || hsv.x > 0.4) || (hsv.y < 0.25) ){
        outputColor = texture(tex0, p);
        return;
    }
    
    if(mode<=2){ //cartesian
        if(mode==0 || mode==1)
//            p.x += intensity * HEIGHT * pNoise(p + (evolution)*vec2(WIDTH,WIDTH), f*1000);
            p.x += intensity * HEIGHT * getNoise(vec3(p.x, p.y, evolution) * vec3(f,f,1));
        if(mode==0 || mode==2)
            p.y += intensity * HEIGHT * getNoise(vec3(p.x, p.y, evolution+1) * vec3(f,f,1));
        
        p.x -= WIDTH*step(WIDTH, p.x);
        p.y -= HEIGHT*step(HEIGHT, p.y);
    }else{ // polar
        p.x -= WIDTH/2;
        p.y -= HEIGHT/2;
        float a = atan(p.y,p.x);
        float d = length(p);
        if(mode==3)
            d += intensity * HEIGHT * pNoise(vec2(a, d) + (evolution)*vec2(WIDTH,WIDTH), f);
        if(mode==4)
            a += intensity * 3.14 * pNoise(vec2(d, a) + (evolution)*vec2(WIDTH,WIDTH), f);
        p.x = d*cos(a) + WIDTH/2;
        p.y = d*sin(a) + HEIGHT/2;
    }
    
//    if(hue>0){
//        vec4 cA = texture(tex0, texCoordVarying);
//        vec4 cB = texture(tex0, p);
//        vec3 hA = rgb2hsv(cA);
//        vec3 hB = rgb2hsv(cB);
//        hA.x = hB.x;
//        hA.y = hB.y;
//        outputColor = hsv2rgb(hA);
//    }
//    else
        if (hueZ>0)
    {
        vec4 cB = texture(tex0, p);
        vec3 hB = rgb2hsv(cB);
        hB.x = mod(hB.x + intensity * hueZ  * 5 * (1+cnoise(vec3(p.y, p.x, evolution) * vec3(f,f,1)))/2.0, 1);
        outputColor = hsv2rgb(hB);
    }
    else
        outputColor = texture(tex0, p);
}

