#version 150
precision mediump float;

uniform sampler2DRect tex0;
out vec4 outputColor;
in vec2 texCoordVarying;
uniform float saturation = 1;
uniform float hue = 0;
uniform float value = 1;
uniform float _floor = 0;

vec3 rgb2hsv(vec4 c){
    vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
    vec4 p = mix(vec4(c.bg, K.wz), vec4(c.gb, K.xy), step(c.b, c.g));
    vec4 q = mix(vec4(p.xyw, c.r), vec4(c.r, p.yzx), step(p.x, c.r));
    
    float d = q.x - min(q.w, q.y);
    float e = 1.0e-10;
    return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
}

vec4 hsv2rgb(vec3 c){
    vec4 K = vec4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
    vec3 p = abs(fract(c.xxx + K.xyz) * 6.0 - K.www);
    return vec4(c.z * mix(K.xxx, clamp(p - K.xxx, 0.0, 1.0), c.y), 1);
}

void main(){
    vec2 p = texCoordVarying;
    
    vec4 c = texture(tex0, p);
    
    float a = c.a;
    vec3 h = rgb2hsv(c);
    h.x += hue;
    if(h.x > 1)
        h.x -= 1;
    
    h.y *= saturation;
    if(h.y < 0)
        h.y += abs(saturation);
    
    h.z *= value;
    if(h.z < 0)
        h.z += abs(value);
    
    c = hsv2rgb(h);
    
    c.r += _floor ;
    c.g += _floor ;
    c.b += _floor ;
    
//    if(c.r + c.g + c.b < _floor*3)
//        c = vec4(0,0,0,1);
    
    c.a = a;
    outputColor = c;
}
