#version 150
precision mediump float;

uniform sampler2DRect tex0;
uniform float tintMap[255];
out vec4 outputColor;
in vec2 texCoordVarying;
uniform float thresh = 0.25;

vec3 rgb2hsv(vec4 c){
    vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
    vec4 p = mix(vec4(c.bg, K.wz), vec4(c.gb, K.xy), step(c.b, c.g));
    vec4 q = mix(vec4(p.xyw, c.r), vec4(c.r, p.yzx), step(p.x, c.r));
    
    float d = q.x - min(q.w, q.y);
    float e = 1.0e-10;
    return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
}

vec4 hsv2rgb(vec3 c){
    vec4 K = vec4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
    vec3 p = abs(fract(c.xxx + K.xyz) * 6.0 - K.www);
    return vec4(c.z * mix(K.xxx, clamp(p - K.xxx, 0.0, 1.0), c.y), 1);
}


float map(float value, float min1, float max1, float min2, float max2){

// Convert the current value to a percentage
// 0% - min1, 100% - max1
float perc = (value - min1) / (max1 - min1);

// Do the same operation backwards with min2 and max2
return value = perc * (max2 - min2) + min2;
}

void main(){
    vec4 col = texture(tex0, texCoordVarying);

    vec3 hsv = rgb2hsv(col);
    
    if((thresh > 0 && hsv.y < thresh) || (thresh < 0 && hsv.y > -thresh))
        outputColor = vec4(0,0,0,1);
    else
//        outputColor = vec4(col.r,col.g,col.b,(hsv.y-thresh)/(1-thresh)); //TODO : smoothing
//    outputColor = vec4(col.r,col.g,col.b,map(hsv.y, thresh, thresh+0.2, 0, 1));
    outputColor = col;
}

