#version 150

uniform sampler2DRect tex0;
in vec2 texCoordVarying;
out vec4 outputColor;
uniform float WIDTH=1280;
uniform float HEIGHT=720;
uniform float nb = 30;
uniform int dist = 4;
uniform float _mix = 0.5;

vec3 rgb2hsv(vec4 c){
    vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
    vec4 p = mix(vec4(c.bg, K.wz), vec4(c.gb, K.xy), step(c.b, c.g));
    vec4 q = mix(vec4(p.xyw, c.r), vec4(c.r, p.yzx), step(p.x, c.r));
    
    float d = q.x - min(q.w, q.y);
    float e = 1.0e-10;
    return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
}

vec4 hsv2rgb(vec3 c){
    vec4 K = vec4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
    vec3 p = abs(fract(c.xxx + K.xyz) * 6.0 - K.www);
    return vec4(c.z * mix(K.xxx, clamp(p - K.xxx, 0.0, 1.0), c.y), 1);
}

void main()
{
    vec2 p = texCoordVarying - vec2(WIDTH/2, HEIGHT/2);
    
    vec4 color = vec4(0,0,0,1);
    vec4 c;
    
    for(int i = 0;i<nb;i++){
        c = texture(tex0, p + vec2(WIDTH/2 + (i-nb/2)*dist, HEIGHT/2));
        vec3 hsv = rgb2hsv(c);
        hsv.x = 0.1 * (i+nb/2) / nb;
        c = hsv2rgb(hsv);
        float a = 1 - (i-nb/2) / nb;
        color += a*c * _mix;
//        color = vec4(max(c.r, color.r), max(c.g, color.g), max(c.b, color.b), 1);
    }
    
    c = texture(tex0, p + vec2(WIDTH/2, HEIGHT/2));

//    color = vec4(max(c.r, color.r), max(c.g, color.g), max(c.b, color.b), 1);
    
    color += c;
    color.a = 1;
    
    outputColor = color;
}